<?php

declare(strict_types=1);

namespace Drupal\mcp\Plugin\McpJsonRpc;

use Drupal\jsonrpc\Object\ParameterBag;
use Drupal\mcp\Plugin\McpJsonRpcMethodBase;
use Drupal\mcp\ServerFeatures\Tool;

/**
 * Lists available tools from the server.
 *
 * @JsonRpcMethod(
 *   id = "tools/list",
 *   usage = @Translation("List available tools."),
 *   params = {
 *     "cursor" = @JsonRpcParameterDefinition(
 *       schema={"type"="string"},
 *       required=false,
 *       description=@Translation("An opaque token representing the current
 *       pagination position.")
 *     )
 *   }
 * )
 */
class ToolsList extends McpJsonRpcMethodBase {

  /**
   * {@inheritdoc}
   */
  public function execute(ParameterBag $params) {
    $tools = [];
    foreach ($this->mcpPluginManager->getAvailablePlugins() as $instance) {
      $instanceTools = $instance->getTools();
      $prefixizedTools = array_map(
        fn($tool) => new Tool(
          name: $instance->getPluginId() . '_' . md5($tool->name),
          description: "Original name: $tool->name, Description: $tool->description",
          inputSchema: $tool->inputSchema,
        ),
        $instanceTools
      );
      $tools = array_merge($tools, $prefixizedTools);
    }

    return [
      'tools' => $tools,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function outputSchema() {
    return [
      'type'       => 'object',
      'required'   => ['tools'],
      'properties' => [
        'tools'      => [
          'type'  => 'array',
          'items' => [
            'type'       => 'object',
            'required'   => ['name', 'inputSchema'],
            'properties' => [
              'name'        => [
                'type'        => 'string',
                'description' => 'The name of the tool',
              ],
              'description' => [
                'type'        => 'string',
                'description' => 'A human-readable description of the tool',
              ],
              'inputSchema' => [
                'type'       => 'object',
                'required'   => ['type'],
                'properties' => [
                  'type' => ['const' => 'object'],
                ],
              ],
            ],
          ],
        ],
        'nextCursor' => [
          'type'        => 'string',
          'description' => 'Token for the next page of results',
        ],
      ],
    ];
  }

}
